String.prototype.replaceAll = function (oldValue, newValue) {
    return this.replace(new RegExp(oldValue, "gm"), newValue);
};

Date.prototype.format = function (format) {
    const dateMap = new Map([
        ['y+', this.getFullYear()],
        ['M+', this.getMonth() + 1],
        ['d+', this.getDate()],
        ['h+', this.getHours()],
        ['m+', this.getMinutes()],
        ['s+', this.getSeconds()],
        ['q+', Math.floor((this.getMonth() + 3) / 3)],
        ['ms+', this.getMilliseconds()]
    ]);
    for (const [key, value] of dateMap) {
        format = format.replace(new RegExp(key), value.toString().length === 1 ? `0${value}` : value);
    }
    return format;
};

const VuePlugin = {};
VuePlugin.install = function (Vue) {
    Vue.mixin({
        created() {

        },
        mounted() {
        },
        computed: {},
        methods: {
            $link(item) {
                this.$router.push(item);
            },
            $toFormData(obj) {
                const formData = new FormData();
                for (let key in obj) {
                    formData.append(key, obj[key]);
                }
                return formData;
            },
            $goBack() {
                this.$router.go(-1);
            },
            $timestampToDateTime(timestamp) {
                return new Date(timestamp).format('yyyy-MM-dd hh:mm')
            },
            $timestampToDate(timestamp) {
                return new Date(timestamp).format('yyyy-MM-dd')
            },
            $getRouteParams(key) {
                return eval('this.$route.params.' + key);
            },
            $objectToParams(url = "", arr) {
                if (arr && arr.length > 0) {
                    let i = 0;
                    if (url.indexOf('?') < 0) url += "?"; else i = 1;
                    arr.forEach((obj, index) => {
                        if (obj.query && (obj.data || obj.data.length > 0)) {
                            if (i === 0) {
                                url += `${obj.query}=${obj.data}`;
                                i++;
                            } else if (obj.query) {
                                url += `&${obj.query}=${obj.data}`;
                            }
                        }
                    });
                }
                return url;
            },
            $getScreenWidth() {
                return document.body.clientWidth;
            }
        }
    });

    Vue.prototype.$getUUID = function (len, radix) {
        var chars = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'.split('');
        var uuid = [], i;
        radix = radix || chars.length;

        if (len) {
            // Compact form
            for (i = 0; i < len; i++) uuid[i] = chars[0 | Math.random() * radix];
        } else {
            // rfc4122, version 4 form
            var r;

            // rfc4122 requires these characters
            uuid[8] = uuid[13] = uuid[18] = uuid[23] = '-';
            uuid[14] = '4';

            // Fill in random data.  At i==19 set the high bits of clock sequence as
            // per rfc4122, sec. 4.1.5
            for (i = 0; i < 36; i++) {
                if (!uuid[i]) {
                    r = 0 | Math.random() * 16;
                    uuid[i] = chars[(i == 19) ? (r & 0x3) | 0x8 : r];
                }
            }
        }

        return uuid.join('');
    };


    Vue.prototype.$setTitle = function (title) {
        document.title = title;
        let iframe = document.createElement('iframe');
        iframe.src = '//m.baidu.com/favicon.ico';
        iframe.style.display = 'none';
        iframe.onload = function () {
            setTimeout(function () {
                iframe.remove();
            }, 9)
        };
        document.body.appendChild(iframe);
    };

    Vue.prototype.$getRequestPath = function () {
        let localObj = window.location;
        let contextPath = localObj.pathname.split("/")[1];
        return localObj.protocol + "//" + localObj.host + "/" + contextPath;
    };

    Vue.prototype.$getServerPath = function () {
        let localObj = window.location;
        return localObj.protocol + "//" + localObj.host;
    };


    Vue.prototype.$serialize = function (data, key) {
        let paramStr = "";
        if (data instanceof String || data instanceof Number || data instanceof Boolean) {
            paramStr += "&" + key + "=" + encodeURIComponent(data);
        } else {
            let self = this;
            this.$each(data, function (i) {
                let k = key == null ? i : key + (data instanceof Array ? "[" + i + "]" : "." + i);
                paramStr += '&' + self.$serialize(this, k);
            });
        }
        return paramStr.substr(1);
    };

    Vue.prototype.$each = function (object, callback, args) {
        let name,
            i = 0,
            length = object.length,
            isObj = length === undefined || typeof object === "function";
        if (args) {
            if (isObj) {
                for (name in object) {
                    if (callback.apply(object[name], args) === false) {
                        break;
                    }
                }
            }
            else {
                for (; i < length;) {
                    if (callback.apply(object[i++], args) === false) {
                        break;
                    }
                }
            }
        }
        else {
            if (isObj) {
                for (name in object) {
                    if (callback.call(object[name], name, object[name]) === false) {
                        break;
                    }
                }
            }
            else {
                for (let value = object[0]; i < length && callback.call(value, i, value) !== false; value = object[++i]) {
                }
            }
        }
        return object;
    };

    Vue.prototype.$JSonToParams = function (url, data, serialize) {
        if (serialize) {
            return url + "?" + data;
        } else {
            if (!data || data.length == 0) return url;
            url += "?";
            let i = 0;
            for (let key in data) {
                if (i == 0) {
                    url = url + key + "=" + data[key];
                    i++;
                } else {
                    url = url + "&" + key + "=" + data[key];
                }
            }
            return url;
        }
    };

    Vue.prototype.$onScroll = function (topCallback, bottomCallback) {
        window.onscroll = function () {
            let a = window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight;
            let b = document.documentElement.scrollTop == 0 ? document.body.scrollTop : document.documentElement.scrollTop;
            let c = document.documentElement.scrollTop == 0 ? document.body.scrollHeight : document.documentElement.scrollHeight;
            if (document.body.scrollTop == 0 && document.documentElement.scrollTop == 0) {
                if (typeof topCallback == "function") {
                    topCallback();
                }
            }
            if (a + Math.floor(b) >= c || a + Math.ceil(b) >= c) {
                if (typeof bottomCallback == "function") {
                    bottomCallback();
                }
            }
        };
    };

    Vue.prototype.$getParam = function (key) {
        let reg = new RegExp("(^|&)" + key + "=([^&]*)(&|$)", "i");
        let r = window.location.search.substr(1).match(reg);
        if (r != null) return decodeURI(r[2]);
        return null;
    }
};

export default VuePlugin;
